@@ -3,6 +3,7 @@ import html
 import os
 import sys
 from collections import defaultdict, Counter
+from enum import Enum
 from textwrap import dedent
 from types import FrameType, CodeType, TracebackType
 from typing import (
@@ -46,6 +47,21 @@ Then use Line.render to insert the markers correctly.
 """
 
 
+class BlankLines(Enum):
+    """The values are intended to correspond to the following behaviour:
+    HIDDEN: blank lines are not shown in the output
+    VISIBLE: blank lines are visible in the output
+    SINGLE: any consecutive blank lines are shown as a single blank line
+            in the output. This option requires the line number to be shown.
+            For a single blank line, the corresponding line number is shown.
+            Two or more consecutive blank lines are shown as a single blank
+            line in the output with a custom string shown instead of a
+            specific line number.
+    """
+    HIDDEN = 1
+    VISIBLE = 2
+    SINGLE=3
+
 class Variable(
     NamedTuple('_Variable',
                [('name', str),
@@ -83,7 +99,10 @@ class Source(executing.Source):
     @cached_property
     def pieces(self) -> List[range]:
         if not self.tree:
-            raise AttributeError("This file doesn't contain valid Python, so .pieces doesn't exist")
+            return [
+                range(i, i + 1)
+                for i in range(1, len(self.lines) + 1)
+            ]
         return list(self._clean_pieces())
 
     @cached_property
@@ -168,13 +187,15 @@ class Options:
             after: int = 1,
             include_signature: bool = False,
             max_lines_per_piece: int = 6,
-            pygments_formatter=None
+            pygments_formatter=None,
+            blank_lines = BlankLines.HIDDEN
     ):
         self.before = before
         self.after = after
         self.include_signature = include_signature
         self.max_lines_per_piece = max_lines_per_piece
         self.pygments_formatter = pygments_formatter
+        self.blank_lines = blank_lines
 
     def __repr__(self):
         keys = sorted(self.__dict__)
@@ -199,6 +220,16 @@ class LineGap(object):
 LINE_GAP = LineGap()
 
 
+class BlankLineRange:
+    """
+    Records the line number range for blank lines gaps between pieces.
+    For a single blank line, begin_lineno == end_lineno.
+    """
+    def __init__(self, begin_lineno: int, end_lineno: int):
+        self.begin_lineno = begin_lineno
+        self.end_lineno = end_lineno
+
+
 class Line(object):
     """
     A single line of source code for a particular stack frame.
@@ -290,15 +321,22 @@ class Line(object):
         A list of one or zero RangeInLines for the executing node of this frame.
         The list will have one element if the node can be found and it overlaps this line.
         """
+        return self._raw_executing_node_ranges(
+            self.frame_info._executing_node_common_indent
+        )
+
+    def _raw_executing_node_ranges(self, common_indent=0) -> List[RangeInLine]:
         ex = self.frame_info.executing
         node = ex.node
         if node:
-            rang = self.range_from_node(node, ex)
+            rang = self.range_from_node(node, ex, common_indent)
             if rang:
                 return [rang]
         return []
 
-    def range_from_node(self, node: ast.AST, data: Any) -> Optional[RangeInLine]:
+    def range_from_node(
+        self, node: ast.AST, data: Any, common_indent: int = 0
+    ) -> Optional[RangeInLine]:
         """
         If the given node overlaps with this line, return a RangeInLine
         with the correct start and end and the given data.
@@ -309,14 +347,28 @@ class Line(object):
         if not (start <= self.lineno <= end):
             return None
         if start == self.lineno:
-            range_start = node.first_token.start[1]
+            try:
+                range_start = node.first_token.start[1]
+            except AttributeError:
+                range_start = node.col_offset
         else:
-            range_start = 0
+            range_start = common_indent
 
         if end == self.lineno:
-            range_end = node.last_token.end[1]
+            try:
+                range_end = node.last_token.end[1]
+            except AttributeError:
+                try:
+                    range_end = node.end_col_offset
+                except AttributeError:
+                    return None
         else:
             range_end = len(self.text)
+        if range_start == range_end == 0:
+            # This is an empty line. If it were included, it would result
+            # in a value of zero for the common indentation assigned to
+            # a block of code.
+            return None
 
         return RangeInLine(range_start, range_end, data)
 
@@ -334,7 +386,7 @@ class Line(object):
         If strip_leading_indent is true (the default) then leading spaces
         common to all lines in this frame will be excluded.
         """
-        if pygmented:
+        if pygmented and self.frame_info.scope:
             assert_(not markers, ValueError("Cannot use pygmented with markers"))
             start_line, lines = self.frame_info._pygmented_scope_lines
             result = lines[self.lineno - start_line]
@@ -503,6 +555,7 @@ class FrameInfo(object):
         self.options = options or Options()  # type: Options
         self.source = self.executing.source  # type: Source
 
+
     def __repr__(self):
         return "{self.__class__.__name__}({self.frame})".format(self=self)
 
@@ -551,10 +604,12 @@ class FrameInfo(object):
     @cached_property
     def scope_pieces(self) -> List[range]:
         """
-        All the pieces (ranges of lines) contained in this object's .scope.
+        All the pieces (ranges of lines) contained in this object's .scope,
+        unless there is no .scope (because the source isn't valid Python syntax)
+        in which case it returns all the pieces in the source file, each containing one line.
         """
         if not self.scope:
-            return []
+            return self.source.pieces
 
         scope_start, scope_end = line_range(self.scope)
         return [
@@ -601,9 +656,6 @@ class FrameInfo(object):
         """
         The piece (range of lines) containing the line currently being executed
         by the interpreter in this frame.
-
-        Raises an exception if .scope is None, which usually means the source code
-        for this frame is unavailable.
         """
         return only(
             piece
@@ -642,10 +694,33 @@ class FrameInfo(object):
         return pieces
 
     @cached_property
-    def lines(self) -> List[Union[Line, LineGap]]:
+    def _executing_node_common_indent(self) -> int:
+        """
+        The common minimal indentation shared by the markers intended
+        for an exception node that spans multiple lines.
+
+        Intended to be used only internally.
+        """
+        indents = []
+        lines = [line for line in self.lines if isinstance(line, Line)]
+
+        for line in lines:
+            for rang in line._raw_executing_node_ranges():
+                begin_text = len(line.text) - len(line.text.lstrip())
+                indent = max(rang.start, begin_text)
+                indents.append(indent)
+
+        if len(indents) <= 1:
+            return 0
+
+        return min(indents[1:])
+
+    @cached_property
+    def lines(self) -> List[Union[Line, LineGap, BlankLineRange]]:
         """
         A list of lines to display, determined by options.
-        The objects yielded either have type Line or are the singleton LINE_GAP.
+        The objects yielded either have type Line, BlankLineRange
+        or are the singleton LINE_GAP.
         Always check the type that you're dealing with when iterating.
 
         LINE_GAP can be created in two ways:
@@ -660,19 +735,25 @@ class FrameInfo(object):
         if not pieces:
             return []
 
+        add_empty_lines = self.options.blank_lines in (BlankLines.VISIBLE, BlankLines.SINGLE)
+        prev_piece = None
         result = []
         for i, piece in enumerate(pieces):
             if (
                     i == 1
+                    and self.scope
                     and pieces[0] == self.scope_pieces[0]
                     and pieces[1] != self.scope_pieces[1]
             ):
                 result.append(LINE_GAP)
+            elif prev_piece and add_empty_lines and piece.start > prev_piece.stop:
+                if self.options.blank_lines == BlankLines.SINGLE:
+                    result.append(BlankLineRange(prev_piece.stop, piece.start-1))
+                else:  # BlankLines.VISIBLE
+                    for lineno in range(prev_piece.stop, piece.start):
+                        result.append(Line(self, lineno))
 
-            lines = [
-                Line(self, i)
-                for i in piece
-            ]  # type: List[Line]
+            lines = [Line(self, i) for i in piece]  # type: List[Line]
             if piece != self.executing_piece:
                 lines = truncate(
                     lines,
@@ -680,6 +761,7 @@ class FrameInfo(object):
                     middle=[LINE_GAP],
                 )
             result.extend(lines)
+            prev_piece = piece
 
         real_lines = [
             line
@@ -695,7 +777,6 @@ class FrameInfo(object):
         leading_indent = len(real_lines[0].text) - len(dedented_lines[0])
         for line in real_lines:
             line.leading_indent = leading_indent
-
         return result
 
     @cached_property
